@@ -1,4 +1,4 @@
-# Copyright 2006-2016 by Peter Cock.  All rights reserved.
+# Copyright 2006-2017 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -8,8 +8,7 @@
 
 r"""Sequence input/output as SeqRecord objects.
 
-Bio.SeqIO is also documented at SeqIO_ and by
-a whole chapter in our tutorial:
+Bio.SeqIO is also documented at SeqIO_ and by a whole chapter in our tutorial:
 
   - `HTML Tutorial`_
   - `PDF Tutorial`_
@@ -35,6 +34,16 @@ Note that the parse() function will invoke the relevant parser for the
 format with its default settings.  You may want more control, in which case
 you need to create a format specific sequence iterator directly.
 
+Some of these parsers are wrappers around low-level parsers which build up
+SeqRecord objects for the consistent SeqIO interface. In cases where the
+run-time is critical, such as large FASTA or FASTQ files, calling these
+underlying parsers will be much faster - in this case these generator
+functions which return tuples of strings:
+
+>>> from Bio.SeqIO.FastaIO import SimpleFastaParser
+>>> from Bio.SeqIO.QualityIO import FastqGeneralIterator
+
+
 Input - Single Records
 ----------------------
 If you expect your file to contain one-and-only-one record, then we provide
@@ -260,12 +269,15 @@ File Formats
 When specifying the file format, use lowercase strings.  The same format
 names are also used in Bio.AlignIO and include the following:
 
-    - abif    - Applied Biosystem's sequencing trace format
+    - abi     - Applied Biosystem's sequencing trace format
+    - abi-trim - Same as "abi" but with quality trimming with Mott's algorithm
     - ace     - Reads the contig sequences from an ACE assembly file.
     - embl    - The EMBL flat file format. Uses Bio.GenBank internally.
     - fasta   - The generic sequence file format where each record starts with
       an identifer line starting with a ">" character, followed by
       lines of sequence.
+    - fasta-2line - Stricter interpretation of the FASTA format using exactly
+      two lines per record (no line wrapping).
     - fastq   - A "FASTA like" format used by Sanger which also stores PHRED
       sequence quality values (with an ASCII offset of 33).
     - fastq-sanger - An alias for "fastq" for consistency with BioPerl and EMBOSS
@@ -389,6 +401,7 @@ from . import UniprotIO
 # Most alignment file formats will be handled via Bio.AlignIO
 
 _FormatToIterator = {"fasta": FastaIO.FastaIterator,
+                     "fasta-2line": FastaIO.FastaTwoLineIterator,
                      "gb": InsdcIO.GenBankIterator,
                      "genbank": InsdcIO.GenBankIterator,
                      "genbank-cds": InsdcIO.GenBankCdsFeatureIterator,
@@ -417,7 +430,21 @@ _FormatToIterator = {"fasta": FastaIO.FastaIterator,
                      "abi-trim": AbiIO._AbiTrimIterator,
                      }
 
+_FormatToString = {
+    "fasta": FastaIO.as_fasta,
+    "fasta-2line": FastaIO.as_fasta_2line,
+    "tab": TabIO.as_tab,
+    "fastq": QualityIO.as_fastq,
+    "fastq-sanger": QualityIO.as_fastq,
+    "fastq-solexa": QualityIO.as_fastq_solexa,
+    "fastq-illumina": QualityIO.as_fastq_illumina,
+    "qual": QualityIO.as_qual,
+}
+
+# This could exclude file formats covered by _FormatToString?
+# Right now used in the unit tests as proxy for all supported outputs...
 _FormatToWriter = {"fasta": FastaIO.FastaWriter,
+                   "fasta-2line": FastaIO.FastaTwoLineWriter,
                    "gb": InsdcIO.GenBankWriter,
                    "genbank": InsdcIO.GenBankWriter,
                    "embl": InsdcIO.EmblWriter,
@@ -431,21 +458,24 @@ _FormatToWriter = {"fasta": FastaIO.FastaWriter,
                    "qual": QualityIO.QualPhredWriter,
                    "sff": SffIO.SffWriter,
                    "seqxml": SeqXmlIO.SeqXmlWriter,
+                   "pir": PirIO.PirWriter,
                    }
 
-_BinaryFormats = ["sff", "sff-trim", "abi", "abi-trim"]
+_BinaryFormats = ["sff", "sff-trim", "abi", "abi-trim", "seqxml"]
 
 
 def write(sequences, handle, format):
     """Write complete set of sequences to a file.
 
-        - sequences - A list (or iterator) of SeqRecord objects, or (if using
-          Biopython 1.54 or later) a single SeqRecord.
-        - handle    - File handle object to write to, or filename as string
-          (note older versions of Biopython only took a handle).
-        - format    - lower case string describing the file format to write.
+    Arguments:
+     - sequences - A list (or iterator) of SeqRecord objects, or (if using
+       Biopython 1.54 or later) a single SeqRecord.
+     - handle    - File handle object to write to, or filename as string
+       (note older versions of Biopython only took a handle).
+     - format    - lower case string describing the file format to write.
 
-    You should close the handle after calling this function.
+    Note if providing a file handle, your code should close the handle
+    after calling this function (to ensure the data gets flushed to disk).
 
     Returns the number of records written (as an integer).
     """
@@ -475,8 +505,14 @@ def write(sequences, handle, format):
         mode = 'w'
 
     with as_handle(handle, mode) as fp:
-        # Map the file format to a writer class
-        if format in _FormatToWriter:
+        # Map the file format to a writer function/class
+        if format in _FormatToString:
+            format_function = _FormatToString[format]
+            count = 0
+            for record in sequences:
+                fp.write(format_function(record))
+                count += 1
+        elif format in _FormatToWriter:
             writer_class = _FormatToWriter[format]
             count = writer_class(fp).write_file(sequences)
         elif format in AlignIO._FormatToWriter:
@@ -503,14 +539,15 @@ def write(sequences, handle, format):
 
 
 def parse(handle, format, alphabet=None):
-    r"""Turns a sequence file into an iterator returning SeqRecords.
+    r"""Turn a sequence file into an iterator returning SeqRecords.
 
-        - handle   - handle to the file, or the filename as a string
-          (note older versions of Biopython only took a handle).
-        - format   - lower case string describing the file format.
-        - alphabet - optional Alphabet object, useful when the sequence type
-          cannot be automatically inferred from the file itself
-          (e.g. format="fasta" or "tab")
+    Arguments:
+     - handle   - handle to the file, or the filename as a string
+       (note older versions of Biopython only took a handle).
+     - format   - lower case string describing the file format.
+     - alphabet - optional Alphabet object, useful when the sequence type
+       cannot be automatically inferred from the file itself
+       (e.g. format="fasta" or "tab")
 
     Typical usage, opening a file to read in, and looping over the record(s):
 
@@ -617,14 +654,15 @@ def _force_alphabet(record_iterator, alphabet):
 
 
 def read(handle, format, alphabet=None):
-    """Turns a sequence file into a single SeqRecord.
+    """Turn a sequence file into a single SeqRecord.
 
-        - handle   - handle to the file, or the filename as a string
-          (note older versions of Biopython only took a handle).
-        - format   - string describing the file format.
-        - alphabet - optional Alphabet object, useful when the sequence type
-          cannot be automatically inferred from the file itself
-          (e.g. format="fasta" or "tab")
+    Arguments:
+     - handle   - handle to the file, or the filename as a string
+       (note older versions of Biopython only took a handle).
+     - format   - string describing the file format.
+     - alphabet - optional Alphabet object, useful when the sequence type
+       cannot be automatically inferred from the file itself
+       (e.g. format="fasta" or "tab")
 
     This function is for use parsing sequence files containing
     exactly one record.  For example, reading a GenBank file:
@@ -676,12 +714,13 @@ def read(handle, format, alphabet=None):
 
 
 def to_dict(sequences, key_function=None):
-    """Turns a sequence iterator or list into a dictionary.
+    """Turn a sequence iterator or list into a dictionary.
 
-        - sequences  - An iterator that returns SeqRecord objects,
-          or simply a list of SeqRecord objects.
-        - key_function - Optional callback function which when given a
-          SeqRecord should return a unique key for the dictionary.
+    Arguments:
+     - sequences  - An iterator that returns SeqRecord objects,
+       or simply a list of SeqRecord objects.
+     - key_function - Optional callback function which when given a
+       SeqRecord should return a unique key for the dictionary.
 
     e.g. key_function = lambda rec : rec.name
     or,  key_function = lambda rec : rec.description.split()[0]
@@ -700,7 +739,7 @@ def to_dict(sequences, key_function=None):
     >>> print(sorted(id_dict))
     ['AF297471.1', 'AJ237582.1', 'L31939.1', 'M81224.1', 'X55053.1', 'X62281.1']
     >>> print(id_dict["L31939.1"].description)
-    Brassica rapa (clone bif72) kin mRNA, complete cds.
+    Brassica rapa (clone bif72) kin mRNA, complete cds
 
     A more complex example, using the key_function argument in order to
     use a sequence checksum as the dictionary key:
@@ -739,14 +778,15 @@ def to_dict(sequences, key_function=None):
 def index(filename, format, alphabet=None, key_function=None):
     """Indexes a sequence file and returns a dictionary like object.
 
-        - filename - string giving name of file to be indexed
-        - format   - lower case string describing the file format
-        - alphabet - optional Alphabet object, useful when the sequence type
-          cannot be automatically inferred from the file itself
-          (e.g. format="fasta" or "tab")
-        - key_function - Optional callback function which when given a
-          SeqRecord identifier string should return a unique
-          key for the dictionary.
+    Arguments:
+     - filename - string giving name of file to be indexed
+     - format   - lower case string describing the file format
+     - alphabet - optional Alphabet object, useful when the sequence type
+       cannot be automatically inferred from the file itself
+       (e.g. format="fasta" or "tab")
+     - key_function - Optional callback function which when given a
+       SeqRecord identifier string should return a unique key for the
+       dictionary.
 
     This indexing function will return a dictionary like object, giving the
     SeqRecord objects as values:
@@ -842,7 +882,8 @@ def index(filename, format, alphabet=None, key_function=None):
     to be completely parsed while building the index. Right now this is
     usually avoided.
 
-    See also: Bio.SeqIO.index_db() and Bio.SeqIO.to_dict()
+    See Also: Bio.SeqIO.index_db() and Bio.SeqIO.to_dict()
+
     """
     # Try and give helpful error messages:
     if not isinstance(filename, basestring):
@@ -877,18 +918,19 @@ def index_db(index_filename, filenames=None, format=None, alphabet=None,
     The index is stored in an SQLite database rather than in memory (as in the
     Bio.SeqIO.index(...) function).
 
-        - index_filename - Where to store the SQLite index
-        - filenames - list of strings specifying file(s) to be indexed, or when
-          indexing a single file this can be given as a string.
-          (optional if reloading an existing index, but must match)
-        - format   - lower case string describing the file format
-          (optional if reloading an existing index, but must match)
-        - alphabet - optional Alphabet object, useful when the sequence type
-          cannot be automatically inferred from the file itself
-          (e.g. format="fasta" or "tab")
-        - key_function - Optional callback function which when given a
-          SeqRecord identifier string should return a unique
-          key for the dictionary.
+    Arguments:
+     - index_filename - Where to store the SQLite index
+     - filenames - list of strings specifying file(s) to be indexed, or when
+       indexing a single file this can be given as a string.
+       (optional if reloading an existing index, but must match)
+     - format   - lower case string describing the file format
+       (optional if reloading an existing index, but must match)
+     - alphabet - optional Alphabet object, useful when the sequence type
+       cannot be automatically inferred from the file itself
+       (e.g. format="fasta" or "tab")
+     - key_function - Optional callback function which when given a
+       SeqRecord identifier string should return a unique
+       key for the dictionary.
 
     This indexing function will return a dictionary like object, giving the
     SeqRecord objects as values:
@@ -916,8 +958,9 @@ def index_db(index_filename, filenames=None, format=None, alphabet=None,
     BGZF compressed files are supported, and detected automatically. Ordinary
     GZIP compressed files are not supported.
 
-    See also: Bio.SeqIO.index() and Bio.SeqIO.to_dict(), and the Python module
+    See Also: Bio.SeqIO.index() and Bio.SeqIO.to_dict(), and the Python module
     glob which is useful for building lists of files.
+
     """
     # Try and give helpful error messages:
     if not isinstance(index_filename, basestring):
@@ -940,8 +983,8 @@ def index_db(index_filename, filenames=None, format=None, alphabet=None,
     # Map the file format to a sequence iterator:
     from ._index import _FormatToRandomAccess  # Lazy import
     from Bio.File import _SQLiteManySeqFilesDict
-    repr = "SeqIO.index_db(%r, filenames=%r, format=%r, alphabet=%r, key_function=%r)" \
-               % (index_filename, filenames, format, alphabet, key_function)
+    repr = ("SeqIO.index_db(%r, filenames=%r, format=%r, alphabet=%r, key_function=%r)"
+            % (index_filename, filenames, format, alphabet, key_function))
 
     def proxy_factory(format, filename=None):
         """Given a filename returns proxy object, else boolean if format OK."""
@@ -958,11 +1001,12 @@ def index_db(index_filename, filenames=None, format=None, alphabet=None,
 def convert(in_file, in_format, out_file, out_format, alphabet=None):
     """Convert between two sequence file formats, return number of records.
 
-        - in_file - an input handle or filename
-        - in_format - input file format, lower case string
-        - out_file - an output handle or filename
-        - out_format - output file format, lower case string
-        - alphabet - optional alphabet to assume
+    Arguments:
+     - in_file - an input handle or filename
+     - in_format - input file format, lower case string
+     - out_file - an output handle or filename
+     - out_format - output file format, lower case string
+     - alphabet - optional alphabet to assume
 
     **NOTE** - If you provide an output filename, it will be opened which will
     overwrite any existing file without warning. This may happen if even
@@ -988,14 +1032,12 @@ def convert(in_file, in_format, out_file, out_format, alphabet=None):
     GTTGCTTCTGGCGTGGGTGGGGGGG
     <BLANKLINE>
     """
-    # Hack for SFF, will need to make this more general in future
     if in_format in _BinaryFormats:
         in_mode = 'rb'
     else:
         in_mode = 'rU'
 
-    # Don't open the output file until we've checked the input is OK?
-    if out_format in ["sff", "sff_trim"]:
+    if out_format in _BinaryFormats:
         out_mode = 'wb'
     else:
         out_mode = 'w'
